@@ -4,7 +4,7 @@
 # gli.h
 #
 # copyright (c) 2004, Carter T. Butts <buttsc@uci.edu>
-# Last Modified 8/24/06
+# Last Modified 1/18/09
 # Licensed under the GNU General Public License version 2 (June, 1991)
 #
 # Part of the R/sna package
@@ -30,7 +30,7 @@
 
 /*R-CALLABLE ROUTINES-------------------------------------------------------*/
 
-void brokerage_R(double *g, int *pn, int *cl, double *brok);
+void brokerage_R(double *g, int *pn, int *pm, int *cl, double *brok);
 
 void lubness_con_R(double *g, double *pn, int *r, double *viol);
 
