@@ -4,7 +4,7 @@
 # randomgraph.h
 #
 # copyright (c) 2004, Carter T. Butts <buttsc@uci.edu>
-# Last Modified 3/12/05
+# Last Modified 4/3/09
 # Licensed under the GNU General Public License version 2 (June, 1991)
 #
 # Part of the R/sna package
@@ -23,6 +23,14 @@
 #include <math.h>
 #include <R.h>
 #include <Rmath.h>
+#include <Rdefines.h>
+#include "utils.h"
+
+/*Homogeneity classes for rgbern*/
+#define BERNHOM 0               /*Homogeneous throughout the adjacency matrix*/
+#define BERNROW 1               /*Homogeneous within rows*/
+#define BERNCOL 2               /*Homogeneous within columns*/
+#define BERNHET 3               /*Heteogeneous*/
 
 
 /*INTERNAL ROUTINES---------------------------------------------------------*/
@@ -30,8 +38,14 @@
 
 /*R-CALLABLE ROUTINES-------------------------------------------------------*/
 
+void bn_cftp_R(int *g, int *pn, double *pi, double *sigma, double *rho, double *d, int *pmaxiter);
+
 void bn_mcmc_R(int *g, double *pn, double *pdraws, double *pburn, int *pthin, double *pi, double *sigma, double *rho, double *d);
+
+SEXP rgbern_R(SEXP sn, SEXP stp, SEXP sdirected, SEXP sloops, SEXP spmode);
+
 void udrewire_R(double *g, double *pn, double *pnv, double *pp);
+
 void wsrewire_R(double *gi, double *go, double *pn, double *pnv, double *pp);
 
 
